#include <basic-timer.hpp>

using namespace stm32f072xb;

BasicTimer::BasicTimer(const BasicTimerInit& init) {
	_timer = init.timer;

	enableClock();

	(init.counter == Counter::ENABLE) ? enableCounter() : disableCounter();
	(init.uevUpdate == UevUpdate::ENABLE) ? enableUevUpdate() : disableUevUpdate();
	(init.updateRequestSource == UpdateRequestSource::ENABLE) ? enableUpdateRequestSource()
	                                                          : disableUpdateRequestSource();
	(init.onePulse == OnePulse::ENABLE) ? enableOnePulse() : disableOnePulse();
	(init.autoReload == AutoReload::ENABLE) ? enableAutoReload() : disableAutoReload();

	setInterruptionPriority(init.interruptPriority);
}

BasicTimer::~BasicTimer() {
	stop();
}

void BasicTimer::enableCounter() const {
	SET_BIT(_timer->CR1, TIM_CR1_CEN);
}
void BasicTimer::disableCounter() const {
	CLEAR_BIT(_timer->CR1, TIM_CR1_CEN);
}

void BasicTimer::stop() {
	disableBoardInterruption();
	disableInterruptUpdate();
	disableCounter();
	setCounter(0x0);
	setAutoReload(0x0);
	setPrescaler(0x0);
}

void BasicTimer::enableUevUpdate() const {
	CLEAR_BIT(_timer->CR1, TIM_CR1_UDIS);
} /* For some reason, enable is set on 0, nice quod vivat ex creatores stm32! */

void BasicTimer::disableUevUpdate() const {
	SET_BIT(_timer->CR1, TIM_CR1_UDIS);
}

void BasicTimer::enableUpdateRequestSource() const {
	SET_BIT(_timer->CR1, TIM_CR1_URS);
}

void BasicTimer::disableUpdateRequestSource() const {
	CLEAR_BIT(_timer->CR1, TIM_CR1_URS);
}

void BasicTimer::enableOnePulse() const {
	SET_BIT(_timer->CR1, TIM_CR1_OPM);
}

void BasicTimer::disableOnePulse() const {
	CLEAR_BIT(_timer->CR1, TIM_CR1_OPM);
}

void BasicTimer::enableAutoReload() const {
	SET_BIT(_timer->CR1, TIM_CR1_ARPE);
}

void BasicTimer::disableAutoReload() const {
	CLEAR_BIT(_timer->CR1, TIM_CR1_ARPE);
}

void BasicTimer::enableInterruptUpdate() const {
	SET_BIT(_timer->DIER, TIM_DIER_UIE);
}

void BasicTimer::disableInterruptUpdate() const {
	CLEAR_BIT(_timer->DIER, TIM_DIER_UIE);
}

void BasicTimer::enableClock() const {
	(_timer == TIM6) ? SET_BIT(RCC->APB1ENR, RCC_APB1ENR_TIM6EN) : SET_BIT(RCC->APB1ENR, RCC_APB1ENR_TIM7EN);
}

void BasicTimer::setInterruptionPriority(const uint32_t priority) const {
	(_timer == TIM6) ? NVIC_SetPriority(TIM6_DAC_IRQn, priority) : NVIC_SetPriority(TIM7_IRQn, priority);
}

void BasicTimer::disableBoardInterruption() const {
	(_timer == TIM6) ? NVIC_DisableIRQ(TIM6_DAC_IRQn) : NVIC_DisableIRQ(TIM7_IRQn);
}

void BasicTimer::enableBoardInterruption() const {
	(_timer == TIM6) ? NVIC_EnableIRQ(TIM6_DAC_IRQn) : NVIC_EnableIRQ(TIM7_IRQn);
}

void BasicTimer::setMasterModeSelection(const MasterModeSelection& masterModeSelection) const {
	switch (masterModeSelection) {
		case MasterModeSelection::RESET:
			CLEAR_BIT(_timer->CR2, TIM_CR2_MMS_0);
			CLEAR_BIT(_timer->CR2, TIM_CR2_MMS_1);
			break;

		case MasterModeSelection::ENABLE:
			SET_BIT(_timer->CR2, TIM_CR2_MMS_0);
			CLEAR_BIT(_timer->CR2, TIM_CR2_MMS_1);
			break;

		case MasterModeSelection::UPDATE:
			CLEAR_BIT(_timer->CR2, TIM_CR2_MMS_0);
			SET_BIT(_timer->CR2, TIM_CR2_MMS_1);
			break;
	}
}

void BasicTimer::updateRegisters() {
	disableInterruptUpdate();
	disableBoardInterruption();

	SET_BIT(_timer->EGR, TIM_EGR_UG);
	CLEAR_BIT(_timer->SR, TIM_SR_UIF);

	enableInterruptUpdate();
	enableBoardInterruption();
}

void BasicTimer::start(const uint16_t autoReloadValue, const uint16_t prescalerValue) {
	setAutoReload(autoReloadValue);
	setPrescaler(prescalerValue);
	enableInterruptUpdate();
	enableBoardInterruption();
	enableCounter();
}

const TIM_TypeDef* BasicTimer::getTimer() const {
	return _timer;
}

uint16_t BasicTimer::getCounter() const {
	return _timer->CNT;
}

uint16_t BasicTimer::getPrescaler() const {
	return _timer->PSC;
}

uint16_t BasicTimer::getAutoReload() const {
	return _timer->ARR;
}

void BasicTimer::setCounter(const uint16_t counterValue) {
	WRITE_REG(_timer->CNT, counterValue);
	updateRegisters();
}

void BasicTimer::setPrescaler(const uint16_t prescaler) {
	_prescaler = (prescaler != 0) ? prescaler : (prescaler - 1);
	WRITE_REG(_timer->PSC, _prescaler);
	updateRegisters();
}

void BasicTimer::setAutoReload(const uint16_t autoReload) {
	_autoReload = (autoReload != 0) ? autoReload : (autoReload - 1);
	WRITE_REG(_timer->ARR, _autoReload);
	updateRegisters();
}
