#include "abstract-uart.hpp"

using namespace stm32f072xb;

AbstractUart::AbstractUart(USART_TypeDef* uart, AbstractUartInit uartInit) : _uart(uart), _uartInit(uartInit) {}

AbstractUart::~AbstractUart() {
	CLEAR_BIT(_uart->CR1, USART_CR1_UE);  // desactivate USART
}

void AbstractUart::setupRegister() {
	WRITE_REG(_uart->CR1, 0x0);
	setWordLength(_uartInit.wordLength);  // bits for words (p.732)
	setStopBits(_uartInit.stopBits);      // stop bit (p.737)
	enableUart();                         // activate USART
	setBaudRate(_uartInit.baudRate);
	SET_BIT(_uart->CR1, USART_CR1_RXNEIE);
}

char AbstractUart::readWord() {
	while (!(_uart->ISR & USART_ISR_RXNE)) {
		if (_uart->ISR & USART_ISR_ORE) {
			_uart->ICR |= USART_ICR_ORECF;
			break;
		}
	}
	return (uint16_t)(_uart->RDR);
}

void AbstractUart::writeWord(uint16_t word) {
	while (!((USART_ISR_TXE & _uart->ISR)))
		;
	_uart->TDR = (char)(word);
	while (!(USART_ISR_TC & _uart->ISR))
		;
}

void AbstractUart::setWordLength(WordLength wordLength) {
	switch (wordLength) {
		case WordLength::HEIGHT_BITS:
			CLEAR_BIT(_uart->CR1, USART_CR1_M0 | USART_CR1_M1);
			break;
		case WordLength::NINE_BITS:
			SET_BIT(_uart->CR1, USART_CR1_M0);
			CLEAR_BIT(_uart->CR1, USART_CR1_M1);
			break;
		case WordLength::SEVEN_BITS:
			CLEAR_BIT(_uart->CR1, USART_CR1_M0);
			SET_BIT(_uart->CR1, USART_CR1_M1);
			break;
	}
}

void AbstractUart::setStopBits(StopBits stopBits) {
	switch (stopBits) {
		case StopBits::ONE_BIT:
			CLEAR_BIT(_uart->CR2, USART_CR2_STOP);
			break;
		case StopBits::HALF_BIT:
			SET_BIT(_uart->CR2, USART_CR2_STOP_0);
			CLEAR_BIT(_uart->CR2, USART_CR2_STOP_1);
			break;
		case StopBits::TWO_BITS:
			CLEAR_BIT(_uart->CR2, USART_CR2_STOP_0);
			SET_BIT(_uart->CR2, USART_CR2_STOP_1);
			break;
		case StopBits::ONE_AND_HALF_BITS:
			SET_BIT(_uart->CR2, USART_CR2_STOP);
			break;
	}
}

void AbstractUart::setBaudRate(uint32_t baudRate) {
	const uint32_t baudRateRegister = 8000000 / baudRate;
	WRITE_REG(_uart->BRR, baudRateRegister);  // using the baud rate equation at p.703
	                                          // and the baud rate register p.743
}

void AbstractUart::enableUart() {
	SET_BIT(_uart->CR1, USART_CR1_UE);  // activate USART
	enableReception();
	enableTransmitter();
}

void AbstractUart::enableTransmitter() {
	SET_BIT(_uart->CR1, USART_CR1_TE);  // enable transmitter (p.734)
}

void AbstractUart::enableReception() {
	SET_BIT(_uart->CR1, USART_CR1_RE);  // enable Reception (p.734)
}
